searchData={"items":[{"type":"module","title":"XmlQuery","doc":"A concise API for querying XML. XML parsing is handled by Erlang/OTP’s built-in\n[xmerl](https://www.erlang.org/doc/man/xmerl) library.\n\nWe created a related library called [HtmlQuery](https://hexdocs.pm/html_query/readme.html) which has the same API but\nis used for querying HTML. You can read more about them in\n[Querying HTML and XML in Elixir with HtmlQuery and XmlQuery](https://eahanson.com/articles/html-query-xml-query).","ref":"XmlQuery.html"},{"type":"module","title":"Data types - XmlQuery","doc":"All functions accept XML in the form of a string, an `Xmerl.xml_attribute`, an `Xmerl.xml_document`, an\n`Xmerl.xml_element`, an `Xmler.xml_text`, an `XmlQuery.Element`, or anything that implements the `String.Chars`\nprotocol.","ref":"XmlQuery.html#module-data-types"},{"type":"module","title":"Query functions - XmlQuery","doc":"| `all/2`   | return all elements matching the selector                   |\n| `find/2`  | return the first element that matches the selector          |\n| `find!/2` | return the only element that matches the selector, or raise |","ref":"XmlQuery.html#module-query-functions"},{"type":"module","title":"Extraction functions - XmlQuery","doc":"| `attr/2` | returns the attribute value as a string      |\n| `text/1` | returns the text contents as a single string |","ref":"XmlQuery.html#module-extraction-functions"},{"type":"module","title":"Parsing & utility functions - XmlQuery","doc":"| `parse/1` | parses XML into an `XmlQuery.Element`, `XmlQuery.Attribute`, or XmlQuery.Text.t() |\n| `pretty/1` | prettifies XML |","ref":"XmlQuery.html#module-parsing-utility-functions"},{"type":"module","title":"Alias - XmlQuery","doc":"If you use XmlQuery a lot, you may want to alias it to the recommended shortcut \"Xq\":\n\n```elixir\nalias XmlQuery, as: Hq\n```","ref":"XmlQuery.html#module-alias"},{"type":"function","title":"XmlQuery.all/2","doc":"Finds all elements in an XML document that match `xpath`, returning a list of records.\nDepending on the given xpath, the type of the record may be different.\n\n``` elixir\niex> xml = ~s|       |\niex> XmlQuery.all(xml, \"//fruit\") |> Enum.map(&to_string/1)\n[\" \",\n \" \"]\n```","ref":"XmlQuery.html#all/2"},{"type":"function","title":"XmlQuery.attr/2","doc":"Returns the value of `attr` from the outermost element of `xml`.\n\n``` elixir\niex> xml = ~s|       |\niex> XmlQuery.attr(xml, :id)\n\"123\"\n```","ref":"XmlQuery.html#attr/2"},{"type":"function","title":"XmlQuery.find/2","doc":"Finds the first element, attribute, or element text in `xml` that matches `xpath`.\n\n``` elixir\niex> alias XmlQuery, as: Xq\niex> xml = \"\"\"\n...> <?xml version=\"1.0\"?>\n...>     \n...> \"\"\"\niex> %Xq.Element{name: :child, attributes: [%Xq.Attribute{value: ~c\"oldest\"}]} = Xq.find(xml, \"//child\")\n\n```","ref":"XmlQuery.html#find/2"},{"type":"function","title":"XmlQuery.find!/2","doc":"Like `find/2` but raises unless exactly one node is found.","ref":"XmlQuery.html#find!/2"},{"type":"macro","title":"XmlQuery.is_xml_struct/1","doc":"","ref":"XmlQuery.html#is_xml_struct/1"},{"type":"function","title":"XmlQuery.parse/1","doc":"Parses an XML document using `:xmerl_scan.string/2`, returning an `XmlQuery.Element` struct.\n\nGiven an xml tuple that has already been created by `:xmerl`, wraps the tuple in an\n`XmlQuery`-specific struct.\n\n``` elixir\niex> xml = \"\"\"\n...> <?xml version=\"1.0\"?>\n...>  \n...> \"\"\"\niex> %Xq.Element{name: :root} = XmlQuery.parse(xml)\n\niex> xml = \"\"\"\n...> <?xml version=\"1.0\"?>\n...>  \n...> \"\"\"\niex> %Xq.Attribute{name: :property, value: ~c\"root-value\"} = XmlQuery.find(xml, \"//root/@property\") |> XmlQuery.parse()\n\n```","ref":"XmlQuery.html#parse/1"},{"type":"function","title":"XmlQuery.pretty/1","doc":"Returns `xml` as a prettified string.\n\nElements and text nodes are sorted and indented relative to parent elements.","ref":"XmlQuery.html#pretty/1"},{"type":"function","title":"XmlQuery.text/1","doc":"Returns the text value of `xml`.\n\n``` elixir\niex> xml = \"  Alice  A.  Aliceston  \"\niex> XmlQuery.text(xml)\n\"Alice A. Aliceston\"\niex> xml |> XmlQuery.find(\"//name/first\") |> XmlQuery.text()\n\"Alice\"\n```","ref":"XmlQuery.html#text/1"},{"type":"type","title":"XmlQuery.xml/0","doc":"","ref":"XmlQuery.html#t:xml/0"},{"type":"type","title":"XmlQuery.xml_attribute/0","doc":"","ref":"XmlQuery.html#t:xml_attribute/0"},{"type":"type","title":"XmlQuery.xml_binary/0","doc":"","ref":"XmlQuery.html#t:xml_binary/0"},{"type":"type","title":"XmlQuery.xml_document/0","doc":"","ref":"XmlQuery.html#t:xml_document/0"},{"type":"type","title":"XmlQuery.xml_element/0","doc":"","ref":"XmlQuery.html#t:xml_element/0"},{"type":"type","title":"XmlQuery.xml_text/0","doc":"","ref":"XmlQuery.html#t:xml_text/0"},{"type":"type","title":"XmlQuery.xpath/0","doc":"","ref":"XmlQuery.html#t:xpath/0"},{"type":"module","title":"XmlQuery.Attribute","doc":"","ref":"XmlQuery.Attribute.html"},{"type":"function","title":"XmlQuery.Attribute.new/1","doc":"","ref":"XmlQuery.Attribute.html#new/1"},{"type":"function","title":"XmlQuery.Attribute.pretty/1","doc":"","ref":"XmlQuery.Attribute.html#pretty/1"},{"type":"type","title":"XmlQuery.Attribute.t/0","doc":"","ref":"XmlQuery.Attribute.html#t:t/0"},{"type":"module","title":"XmlQuery.Element","doc":"","ref":"XmlQuery.Element.html"},{"type":"function","title":"XmlQuery.Element.new/1","doc":"","ref":"XmlQuery.Element.html#new/1"},{"type":"function","title":"XmlQuery.Element.pretty/1","doc":"","ref":"XmlQuery.Element.html#pretty/1"},{"type":"type","title":"XmlQuery.Element.t/0","doc":"","ref":"XmlQuery.Element.html#t:t/0"},{"type":"module","title":"XmlQuery.Element.PrettyFormatter","doc":"","ref":"XmlQuery.Element.PrettyFormatter.html"},{"type":"function","title":"XmlQuery.Element.PrettyFormatter.#element#/5","doc":"","ref":"XmlQuery.Element.PrettyFormatter.html##element#/5"},{"type":"function","title":"XmlQuery.Element.PrettyFormatter.#text#/1","doc":"","ref":"XmlQuery.Element.PrettyFormatter.html##text#/1"},{"type":"function","title":"XmlQuery.Element.PrettyFormatter.#xml-inheritance#/0","doc":"","ref":"XmlQuery.Element.PrettyFormatter.html##xml-inheritance#/0"},{"type":"module","title":"XmlQuery.Text","doc":"","ref":"XmlQuery.Text.html"},{"type":"function","title":"XmlQuery.Text.new/1","doc":"","ref":"XmlQuery.Text.html#new/1"},{"type":"function","title":"XmlQuery.Text.pretty/1","doc":"","ref":"XmlQuery.Text.html#pretty/1"},{"type":"type","title":"XmlQuery.Text.t/0","doc":"","ref":"XmlQuery.Text.html#t:t/0"},{"type":"module","title":"XmlQuery.Xmerl","doc":"","ref":"XmlQuery.Xmerl.html"},{"type":"macro","title":"XmlQuery.Xmerl.xmlAttribute/1","doc":"","ref":"XmlQuery.Xmerl.html#xmlAttribute/1"},{"type":"macro","title":"XmlQuery.Xmerl.xmlAttribute/2","doc":"","ref":"XmlQuery.Xmerl.html#xmlAttribute/2"},{"type":"macro","title":"XmlQuery.Xmerl.xmlDocument/1","doc":"","ref":"XmlQuery.Xmerl.html#xmlDocument/1"},{"type":"macro","title":"XmlQuery.Xmerl.xmlDocument/2","doc":"","ref":"XmlQuery.Xmerl.html#xmlDocument/2"},{"type":"macro","title":"XmlQuery.Xmerl.xmlElement/1","doc":"","ref":"XmlQuery.Xmerl.html#xmlElement/1"},{"type":"macro","title":"XmlQuery.Xmerl.xmlElement/2","doc":"","ref":"XmlQuery.Xmerl.html#xmlElement/2"},{"type":"macro","title":"XmlQuery.Xmerl.xmlText/1","doc":"","ref":"XmlQuery.Xmerl.html#xmlText/1"},{"type":"macro","title":"XmlQuery.Xmerl.xmlText/2","doc":"","ref":"XmlQuery.Xmerl.html#xmlText/2"},{"type":"type","title":"XmlQuery.Xmerl.xml_attribute/0","doc":"","ref":"XmlQuery.Xmerl.html#t:xml_attribute/0"},{"type":"type","title":"XmlQuery.Xmerl.xml_document/0","doc":"","ref":"XmlQuery.Xmerl.html#t:xml_document/0"},{"type":"type","title":"XmlQuery.Xmerl.xml_element/0","doc":"","ref":"XmlQuery.Xmerl.html#t:xml_element/0"},{"type":"type","title":"XmlQuery.Xmerl.xml_text/0","doc":"","ref":"XmlQuery.Xmerl.html#t:xml_text/0"},{"type":"exception","title":"XmlQuery.QueryError","doc":"An exception raised when unable to find an XML element.","ref":"XmlQuery.QueryError.html"},{"type":"extras","title":"XmlQuery","doc":"# XmlQuery\n\nA concise API for querying XML using xpath. There are just 5 main functions:\n`all/2`, `find/2` and `find!/2` for finding things, plus `attr/2` and `text/1` for extracting\ninformation. There are also a handful of other useful functions, referenced below and described in detail in\nthe [module docs](https://hexdocs.pm/xml_query/XmlQuery.html). XML parsing is handled by Erlang/OTP’s built-in\n[xmerl](https://www.erlang.org/doc/man/xmerl) library.\n\nThe input can be:\n\n* A string of XML.\n* An `Xmerl.xml_attribute()`, `Xmerl.xml_document()`, `Xmerl.xml_element()`, or `Xmerl.xml_text()`.\n* An `XmlQuery.Element` struct.\n* Anything that implements the `String.Chars` protocol.\n\nWe created a related library called [HtmlQuery](https://hexdocs.pm/html_query/readme.html) which has the same API but\nis used for querying HTML. You can read more about them in\n[Querying HTML and XML in Elixir with HtmlQuery and XmlQuery](https://eahanson.com/articles/html-query-xml-query).\n\nThis library is MIT licensed and is part of a growing number of Elixir open source libraries published at\n[github.com/synchronal](https://github.com/synchronal#elixir).","ref":"readme.html"},{"type":"extras","title":"Installation - XmlQuery","doc":"```elixir\ndef deps do\n  [\n    {:xml_query, \"~> 0.2.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage - XmlQuery","doc":"Detailed docs are in the [XmlQuery module docs](https://hexdocs.pm/xml_query/XmlQuery.html); a quick usage\noverview follows.\n\nWe typically alias `XmlQuery` to `Xq`:\n\n```elixir\nalias XmlQuery, as: Xq\n```\n\nThe rest of these examples use the following XML:\n\n```elixir\nxml = \"\"\"\n<?xml version=\"1.0\"?>\n \n   \n   \n     \n       \n         Leg \n         Lion \n         Blue \n       \n     \n   \n \n\"\"\"\n```","ref":"readme.html#usage"},{"type":"extras","title":"Querying - XmlQuery","doc":"Query functions use XPath selector strings for finding nodes. The\n[Devhints XPath cheatsheet](https://devhints.io/xpath) is a helpful XPath reference.\n\n```elixir\nXq.find!(xml, \"//toy/animal\")\n```\n\nThe result of a query is an `XmlQuery.Element` struct, a list of `XmlQuery.Element` structs, an `XmlQuery.Attribute`\nstruct, or an `XmlQuery.Text` struct. All of these structs implement `String.Chars` so you can convert them to strings\nwith `to_string/1`:\n\n```elixir\nXq.find!(xml, \"//toy/animal\") |> to_string() # returns \" Lion \"\n```","ref":"readme.html#querying"},{"type":"extras","title":"Finding - XmlQuery","doc":"```elixir\nXq.all(xml, \"//child\") # returns a list of all the   elements\nXq.find(xml, ~s|//child[@name=\"Alice\"]|) # returns the   with name \"Alice\"\nXq.find!(xml, ~s|//child[@name=\"foo\"]|) # raises because no such element exists\n```\n\nSee the [module docs](https://hexdocs.pm/xml_query/XmlQuery.html) for more details.","ref":"readme.html#finding"},{"type":"extras","title":"Extracting - XmlQuery","doc":"`text/1` is the simplest extraction function:\n\n```elixir\nxml |> Xq.find!(\"//toy\") |> Xq.text() # returns \"Leg Lion Blue\"\n```\n\n`attr/2` returns the value of an attribute:\n\n```elixir\nxml |> Xq.find!(~s|//toy[@part-number=\"voltr-123\"]|) |> Xq.attr(\"name\") # returns \"Voltron\"\n```\n\nTo extract data from multiple XML nodes, we found that it is clearer to compose multiple functions rather than to\nhave a more complicated API:\n\n```elixir\nxml |> Xq.all(\"//toy/*\") |> Enum.map(&Xq.text/1) # returns [\"Leg\", \"Lion\", \"Blue\"]\nxml |> Xq.all(\"//child\") |> Enum.map(&Xq.attr(&1, \"age\")) # returns [\"12\", \"9\"]\n```\n\nSee the [module docs](https://hexdocs.pm/xml_query/XmlQuery.html) for more details.","ref":"readme.html#extracting"},{"type":"extras","title":"Parsing - XmlQuery","doc":"`parse/1` parses an XML document and returns an `XmlQuery.Element` struct. It is rarely needed since all the XmlQuery\nfunctions will parse XML if needed. See the [module docs](https://hexdocs.pm/xml_query/XmlQuery.html) for more details.","ref":"readme.html#parsing"},{"type":"extras","title":"Utilities - XmlQuery","doc":"`pretty/1` formats XML in a human-friendly format. See the [module docs](https://hexdocs.pm/xml_query/XmlQuery.html)\nfor more details.","ref":"readme.html#utilities"},{"type":"extras","title":"Development - XmlQuery","doc":"```shell\nbin/dev/doctor\nbin/dev/update\nbin/dev/audit\nbin/dev/shipit\n```","ref":"readme.html#development"},{"type":"extras","title":"References - XmlQuery","doc":"- https://gist.github.com/sasa1977/5967224\n- https://medium.com/erlang-battleground/the-hidden-xml-simplifier-a5f66e10c928","ref":"readme.html#references"},{"type":"extras","title":"LICENSE","doc":"Copyright 2023 synchronal.dev\n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the \"Software\"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.35.1"}}